@@ -26,22 +26,19 @@
 
 using namespace pan;
 
-StringView
+const StringView&
 ArticleFilter :: get_header (const Article& a, const Quark& header_name) const
 {
-  StringView value;
+  static const StringView empty;
 
-  if      (header_name == subject)    value = a.subject.c_str();
-  else if (header_name == from)       value = a.author.c_str();
-  else if (header_name == message_Id) value = a.message_id.c_str();
-  else if (header_name == message_ID) value = a.message_id.c_str();
-  else {
-    std::cerr << LINE_ID << ' ' << PACKAGE_STRING << " is misparsing \"" << header_name << "\".\n"
-              << "Please file a bug report to http://bugzilla.gnome.org/enter_bug.cgi?product=Pan\n";
-    value = "";
-  }
+  if (header_name == subject)    return a.subject.to_view ();
+  if (header_name == from)       return a.author.to_view ();
+  if (header_name == message_Id) return a.message_id.to_view ();
+  if (header_name == message_ID) return a.message_id.to_view ();
 
-  return value;
+  std::cerr << LINE_ID << ' ' << PACKAGE_STRING << " is misparsing \"" << header_name << "\".\n"
+            << "Please file a bug report to http://bugzilla.gnome.org/enter_bug.cgi?product=Pan\n";
+  return empty;
 }
 
 
