@@ -62,6 +62,7 @@ namespace pan
       void open_draft ();
       void send_now ();
       void close_window ();
+      void set_wrap_mode (bool wrap);
 
     private:
       void done_sending_message (GMimeMessage*, bool);
@@ -77,6 +78,7 @@ namespace pan
 
     private:
       virtual void on_progress_finished (Progress&, int status);
+      virtual void on_progress_error (Progress&, const StringView&);
 
     private:
       Data& _data;
@@ -115,7 +117,14 @@ namespace pan
       void apply_profile_to_headers ();
       enum Mode { DRAFTING, POSTING };
       GMimeMessage * new_message_from_ui (Mode mode);
+      Profile get_current_profile ();
       bool check_message (const Quark& server, GMimeMessage*);
+      bool check_charset ();
+
+    private:
+      std::string get_body () const;
+      gulong body_view_realized_handler;
+      static void body_view_realized (GtkWidget*, gpointer);
 
     private:
       unsigned long _group_entry_changed_id;
