@@ -191,7 +191,7 @@ TaskArticle :: use_nntp (NNTP * nntp)
     // maybe our needs-work list is out of date, so call update_work()
     // before giving the nntp back.
     update_work ();
-    check_in (nntp, true);
+    check_in (nntp, OK);
   }
   else
   {
@@ -234,8 +234,9 @@ TaskArticle :: on_nntp_line  (NNTP               * nntp,
 }
 
 void
-TaskArticle :: on_nntp_done  (NNTP    * nntp,
-                              Health    health)
+TaskArticle :: on_nntp_done  (NNTP             * nntp,
+                              Health             health,
+                              const StringView & response)
 {
   // find the Needed using this nntp...
   needed_t::iterator it;
@@ -252,11 +253,11 @@ TaskArticle :: on_nntp_done  (NNTP    * nntp,
     ++_stats[nntp->_server];
   }
 
-  if (health == FAIL) // if server doesn't have that article...
+  if (health == COMMAND_FAILED) // if server doesn't have that article...
     it->xref.remove_server (nntp->_server);
 
   // if we got it or still have other options, we're okay
-  _state.set_health (health==FAIL && it->xref.empty() ? FAIL : OK);
+  _state.set_health (health==COMMAND_FAILED && it->xref.empty() ? COMMAND_FAILED : OK);
 
   if (health==OK)
     _needed.erase (it);
@@ -267,7 +268,7 @@ TaskArticle :: on_nntp_done  (NNTP    * nntp,
   }
 
   update_work ();
-  check_in (nntp, health!=RETRY);
+  check_in (nntp, health);
 }
 
 namespace
@@ -326,12 +327,11 @@ TaskArticle :: on_finished ()
     {
       UUSetMsgCallback (NULL, uu_log);
       UUSetOption (UUOPT_DESPERATE, 1, NULL); // keep incompletes -- they're still useful to par2
-      UUSetOption (UUOPT_FAST, 1, NULL); // yes, each input file is exactly one newsgroup posting
 
       int i (0);
       foreach_const (ArticleCache::strings_t, filenames, it) {
         if ((res = UULoadFileWithPartNo (const_cast<char*>(it->c_str()), 0, 0, ++i)) != UURET_OK)
-          Log::add_err_va (_("Couldn't load \"%s\": %s"), it->c_str(),
+          Log::add_err_va (_("Error reading from %s: %s"), it->c_str(),
             (res==UURET_IOERR) ?  file::pan_strerror (UUGetOption (UUOPT_ERRNO, NULL, NULL, 0)) : UUstrerror(res));
       }
 
@@ -372,9 +372,12 @@ TaskArticle :: on_finished ()
         }
 
         // decode the file...
-        if ((res = UUDecodeFile (item, fname)) == UURET_OK)
+        if ((res = UUDecodeFile (item, fname)) == UURET_OK) {
           Log::add_info_va (_("Saved \"%s\""), fname);
-        else {
+        } else if (res == UURET_NODATA) {
+          // silently let this error by... user probably tried to
+          // save attachements on a text-only post
+        } else {
           const int the_errno (UUGetOption (UUOPT_ERRNO, NULL, NULL, 0));
           if (res==UURET_IOERR && the_errno==ENOSPC)
             Log::add_urgent_va (_("Error saving \"%s\":\n%s. %s"), fname, file::pan_strerror(the_errno), "ENOSPC");
